﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ConditionActionEncoding {
  [Item("RandomActionSelector", "A random action selector.")]
  [StorableClass]
  public class RandomActionSelector : OldActionSelector {
    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }

    [StorableConstructor]
    protected RandomActionSelector(bool deserializing) : base(deserializing) { }
    protected RandomActionSelector(RandomActionSelector original, Cloner cloner) : base(original, cloner) { }
    public RandomActionSelector()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator used for stochastic selection."));
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new RandomActionSelector(this, cloner);
    }

    protected override IAction SelectAction(List<IScope> scopes) {
      IRandom random = RandomParameter.ActualValue;
      int index = random.Next(MatchParameter.ActualValue.Length);
      return MatchParameter.ActualValue[index].Action;
    }
  }
}
